@@ -25,14 +25,15 @@ import sys
 import time
 import traceback
 import base64
-
+import signal
 import asyncio
 import aiohttp.web
 import typing
+import uuid
 
 import ahapi.formdata
 
-__version__ = "0.1.16"
+__version__ = "0.1.18"
 
 
 KNOWN_TEXT_EXTENSIONS = {
@@ -67,7 +68,7 @@ class SimpleServer:
         dir_relative = dirname.replace(self.api_root, "", 1)
         for endpoint_file in os.listdir(dirname):
             endpoint_path = os.path.join(dirname, endpoint_file)
-            if endpoint_file.endswith(".py"):
+            if endpoint_file.endswith(".py") and os.path.isfile(endpoint_path) and endpoint_file != "__init__.py":
                 endpoint = endpoint_file[:-3]
                 modname = ".".join(dir_relative.split("/"))
                 spec = importlib.util.spec_from_file_location(f"{modname}.{endpoint}", endpoint_path)
@@ -91,17 +92,21 @@ class SimpleServer:
         bind_port: int = 8080,
         state: typing.Any = None,
         max_upload: int = ahapi.formdata.AHAPI_MAX_PAYLOAD,
+        log_stdout = False,
+        log_web = True,
     ):
         print("==== Starting HTTP API server... ====")
         self.state = state
         self.handlers: typing.Dict[str, Endpoint] = {}
-        self.server = None
+        self.server: typing.Optional[aiohttp.web.Server] = None
         self.bind_ip = bind_ip
         self.bind_port = bind_port
         self.api_root = api_dir
         self.static_dir = static_dir
         self.max_upload = max_upload
         self.pending_headers: typing.Dict[int, dict] = {}
+        self.log_stdout: bool = log_stdout
+        self.log_web: bool = log_web
 
         # Load each URL endpoint
         self.load_api_dir(api_dir)
@@ -175,7 +180,19 @@ class SimpleServer:
                     del self.pending_headers[hid]
                 exc_type, exc_value, exc_traceback = sys.exc_info()
                 err = "\n".join(traceback.format_exception(exc_type, exc_value, exc_traceback))
-                return aiohttp.web.Response(headers=headers, status=500, text="API error occurred: \n" + err)
+                error_id = ""
+                if self.log_stdout:
+                    error_id = uuid.uuid4()
+                    for line in traceback.format_exception(exc_type, exc_value, exc_traceback):
+                        print(f"[{error_id}] {line}")
+                if self.log_web:
+                    return aiohttp.web.Response(headers=headers, status=500, text="API error occurred: \n" + err)
+                else:
+                    if error_id:
+                        error_code = f"API error occurred! See the log output for more information. Error log ID: {error_id}"
+                    else:
+                        error_code = f"API error occurred!"
+                    return aiohttp.web.Response(headers=headers, status=500, text=error_code)
 
         # Static file handler?
         elif self.static_dir and static_file_path and os.path.isfile(static_file_path):
@@ -203,12 +220,20 @@ class SimpleServer:
         else:
             return aiohttp.web.Response(headers=headers, status=404, text="API Endpoint not found!")
 
-    async def loop(self, forever=True):
+    def sighup(self):
+        print("Stopping AHAPI...")
+        # TODO: More shutdown here?
+        raise aiohttp.web.GracefulExit
+
+    async def loop(self, main_loop=None, forever=True):
         self.server = aiohttp.web.Server(self.handle_request)
-        runner = aiohttp.web.ServerRunner(self.server)
+        runner = aiohttp.web.ServerRunner(self.server, handle_signals=True)
         await runner.setup()
         site = aiohttp.web.TCPSite(runner, self.bind_ip, self.bind_port)
         await site.start()
+        if main_loop is None:  # Assume main event loop if none supplied, for signal handling
+            main_loop = asyncio.get_event_loop()
+        main_loop.add_signal_handler(signal.SIGHUP, self.sighup)
         print("==== HTTP API Server running on %s:%s ====" % (self.bind_ip, self.bind_port))
         if forever:
             while True:
